@@ -34,16 +34,15 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <mach/vm_region.h>
 #include <mach/vm_statistics.h>
 #include <yara/error.h>
+#include <yara/libyara.h>
 #include <yara/mem.h>
 #include <yara/proc.h>
 
-
 typedef struct _YR_PROC_INFO
 {
   task_t task;
 } YR_PROC_INFO;
 
-
 int _yr_process_attach(int pid, YR_PROC_ITERATOR_CTX* context)
 {
   YR_PROC_INFO* proc_info = (YR_PROC_INFO*) yr_malloc(sizeof(YR_PROC_INFO));
@@ -64,7 +63,6 @@ int _yr_process_attach(int pid, YR_PROC_ITERATOR_CTX* context)
   return ERROR_SUCCESS;
 }
 
-
 int _yr_process_detach(YR_PROC_ITERATOR_CTX* context)
 {
   YR_PROC_INFO* proc_info = context->proc_info;
@@ -75,7 +73,6 @@ int _yr_process_detach(YR_PROC_ITERATOR_CTX* context)
   return ERROR_SUCCESS;
 }
 
-
 YR_API const uint8_t* yr_process_fetch_memory_block_data(YR_MEMORY_BLOCK* block)
 {
   YR_PROC_ITERATOR_CTX* context = (YR_PROC_ITERATOR_CTX*) block->context;
@@ -113,7 +110,6 @@ YR_API const uint8_t* yr_process_fetch_memory_block_data(YR_MEMORY_BLOCK* block)
   return context->buffer;
 }
 
-
 YR_API YR_MEMORY_BLOCK* yr_process_get_next_memory_block(
     YR_MEMORY_BLOCK_ITERATOR* iterator)
 {
@@ -126,8 +122,13 @@ YR_API YR_MEMORY_BLOCK* yr_process_get_next_memory_block(
   vm_region_basic_info_data_64_t info;
   vm_size_t size = 0;
 
-  vm_address_t address = (vm_address_t) context->current_block.base +
-                         context->current_block.size;
+  uint64_t current_begin = (vm_address_t) context->current_block.base +
+                           context->current_block.size;
+  vm_address_t address = current_begin;
+  uint64_t max_process_memory_chunk;
+
+  yr_get_configuration_uint64(
+      YR_CONFIG_MAX_PROCESS_MEMORY_CHUNK, &max_process_memory_chunk);
 
   iterator->last_error = ERROR_SUCCESS;
 
@@ -146,18 +147,26 @@ YR_API YR_MEMORY_BLOCK* yr_process_get_next_memory_block(
 
     if (kr == KERN_SUCCESS)
     {
-      context->current_block.base = address;
-      context->current_block.size = size;
+      size_t chunk_size = size - (size_t) (current_begin - address);
+
+      if (((uint64_t) chunk_size) > max_process_memory_chunk)
+      {
+        chunk_size = (size_t) max_process_memory_chunk;
+      }
+
+      context->current_block.base = (size_t) current_begin;
+      context->current_block.size = chunk_size;
 
       return &context->current_block;
     }
 
+    current_begin = address;
+
   } while (kr != KERN_INVALID_ADDRESS);
 
   return NULL;
 }
 
-
 YR_API YR_MEMORY_BLOCK* yr_process_get_first_memory_block(
     YR_MEMORY_BLOCK_ITERATOR* iterator)
 {
